@@ -1,7 +1,7 @@
 /*
  *        JacORB - a free Java ORB
  *
- *   Copyright (C) 1997-98  Gerald Brose.
+ *   Copyright (C) 1997-2000  Gerald Brose.
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Library General Public
@@ -27,52 +27,54 @@ package jacorb.orb;
  * vice versa<p>
  *
  * @author Gerald Brose, FU Berlin
- * @version $Id: Connection.java,v 1.42 2000/04/04 14:05:39 brose Exp $
+ * @version $Id: Connection.java,v 1.60 2000/10/24 12:33:58 brose Exp $
  *
  */
 
-
 import java.io.*;
 import java.net.*;
 import java.util.*;
+
+import jacorb.util.Debug;
 import jacorb.orb.giop.*;
 import jacorb.util.*;
+import jacorb.orb.factory.SocketFactory;
 
-public final class Connection 
-    implements ClientSideConnection, ServerSideConnection
+public class Connection 
 {
     private int id_count = 0;
-    private String connection_info = null;
-    private BufferedInputStream in_stream;
+    protected String connection_info = null;
+    protected InputStream in_stream;
     BufferedOutputStream out_stream;
 
     protected jacorb.orb.ORB orb;
+    protected ConnectionManager manager;
 
     /** write lock */
     public Object writeLock = new Object();
 
-
     // repID -> object
-    private Hashtable objects = new Hashtable();
+    protected Hashtable objects = new Hashtable();
 
-  //contains RequestOutputStreams instead of byte[]
-  private Hashtable buffers = new Hashtable();
+    //contains RequestOutputStreams instead of byte[]
+    protected Hashtable buffers = new Hashtable();
 
-    private Hashtable replies = new Hashtable();
-    private boolean littleEndian;
-    private boolean is_server;
-    private boolean is_ssl;
+    protected Hashtable replies = new Hashtable();
+    protected boolean littleEndian;
+    protected boolean is_server;
 	
     /**
      * Connection OSF character formats, if zero for client side connection
-     * then prefered tcs was not computed yet; use TCS for zero testing.
-     * Note that zero values are invalid when tcsNegotiated=true.
+     * then preferred tcs was not computed yet; use TS for zero testing.
+     * Note that zero values are invalid when tcsNegotiated = true.
      * It's protected because it's directly read by CDR classes defined
      * in this package.
+     *
      * @author devik
      */
 
-    protected int TCS=0,TCSW=0;	
+    protected int TCS = 0, TCSW = 0;	
+
     /**
      * Client's information whether the codeSet has been already sent to server.
      * It's important because there is possibility that we add the context
@@ -89,25 +91,95 @@ public final class Connection
      * @author devik
      */
 
-    private boolean tcsNegotiated=false;
+    private boolean tcsNegotiated = false;
 	
     /**
      * IIOP version active on the channel.
      * @author devik
      */
 
-    protected org.omg.IIOP.Version IIOPVersion=new org.omg.IIOP.Version((byte)1,(byte)0);
+    protected org.omg.IIOP.Version IIOPVersion = 
+	new org.omg.IIOP.Version((byte)1,(byte)0);
 
     /* how many clients use this connection? */
-    private int client_count = 0;
+    protected int client_count = 0;
 
-    private Socket mysock = null;
-    private ReplyReceptor rep;
+    protected Socket mysock = null;
+    protected ReplyReceptor repReceptor;
     private byte [] header = new byte[ Messages.MSG_HEADER_SIZE ];
+    protected SocketFactory socket_factory = null;
+    
+    /** 
+     * dummy constructor
+     */
+
+    public Connection(){}
+
+    /**
+     * class Constructor, establishes a connection over a given socket.
+     * It is called from ConnectionManager.getConnection for clients and
+     * from BasicAdapter for servers
+     *
+     * @param <code>Socket s</code>
+     * @param <code>boolean _is_server</code> - if opened from the server side.<br>
+     * @except <code>java.io.IOException</code>
+     */
+	
+	
+    public Connection ( jacorb.orb.ConnectionManager mgr, 
+                        java.net.Socket s, 
+                        boolean _is_server,
+                        SocketFactory socket_factory )
+        throws IOException
+    {
+        this( mgr, s, _is_server, 
+              new BufferedInputStream( s.getInputStream()),
+              socket_factory );
+    }
+
+
+    public Connection ( jacorb.orb.ConnectionManager mgr,
+                        java.net.Socket s, 
+                        boolean _is_server, 
+                        InputStream in,
+                        SocketFactory socket_factory )
+        throws IOException
+    {
+        this.socket_factory = socket_factory;
+        manager = mgr;
+        this.orb = mgr.getORB();
+        mysock = s;
+        is_server = _is_server;
+
+        in_stream = in;
+        out_stream = 
+            new BufferedOutputStream( mysock.getOutputStream(), 
+                                     Environment.outBufSize() );
+
+        String ip = mysock.getInetAddress().getHostAddress();
+        if ( ip.indexOf('/') > 0 ) 
+            ip = ip.substring( ip.indexOf('/') + 1 );
+
+        String host_and_port = ip + ":"+ mysock.getPort();
+        String ssl = isSSL() ? "SSL " : ""; //bnv
+        connection_info = host_and_port;
+        client_count = 1;
+
+        if ( ! _is_server ) // for clients only:
+        {
+            Debug.output(1, "New " + ssl + "connection to " + host_and_port);
+            repReceptor = new ReplyReceptor( this );
+        }
+        else
+        {
+            Debug.output(1,"Accepted " + ssl + "connection from " + host_and_port);
+        }
+    }
+
 
     ORB getORB()
     {
-	return orb;
+        return orb;
     }
 
 
@@ -116,36 +188,43 @@ public final class Connection
      * connection as negotiated. It's public because it has to be called
      * also from dii.Request.
      */
-
+	
     public void markTcsNegotiated()
     {
-	if(tcsNegotiated) 
-	    return;
-	tcsNegotiated = true;
-	jacorb.util.Debug.output(4,"TCS marked as negotiated");
+        if( tcsNegotiated ) 
+            return;
+        tcsNegotiated = true;
+        Debug.output(4,"TCS marked as negotiated");
     }
 	
     public boolean isTCSNegotiated()
     {
-	return tcsNegotiated;
+        return tcsNegotiated;
     }
 	
     public boolean isSSL()
     {
-	return is_ssl;
+        return socket_factory.isSSL( mysock );
     }
-
+	
+    /** don't use, this is a temporary hack! */
+	
+    public Socket getSocket()
+    {
+        return mysock;
+    }
+		
     protected int selectCodeSet(org.omg.CONV_FRAME.CodeSetComponent scs,
 				int ourNative)
     {
 	// check if we support server's native sets
-	if(scs.native_code_set==ourNative) 
+	if( scs.native_code_set == ourNative ) 
 	    return ourNative;
 		
 	// is our native CS supported at server ?
-	for(int i=0;i<scs.conversion_code_sets.length;i++)
+	for( int i=0; i < scs.conversion_code_sets.length; i++)
 	{
-	    if(scs.conversion_code_sets[i]==ourNative) 
+	    if( scs.conversion_code_sets[i] == ourNative ) 
 		return ourNative;
 	}
 		
@@ -154,46 +233,49 @@ public final class Connection
     }
 
     /**
-     * This code selects appropriate codeset for connection from information
+     * This code selects the appropriate codeset for connection from information
      * contained in some IOR. Code is called by client side connection.
      * Returns true if common codeset was found and so that it should be
-     * send in context.
+     * sent in context.
      */
     protected boolean selectCodeSet(ParsedIOR pior)
     {
-	for(int i=0;i<pior.taggedComponents.length;i++)
+	for(int i = 0; i < pior.taggedComponents.length; i++)
 	{
-	    if(pior.taggedComponents[i].tag != org.omg.IOP.TAG_CODE_SETS.value) 
+	    if( pior.taggedComponents[i].tag != org.omg.IOP.TAG_CODE_SETS.value ) 
 		continue;
 
-	    jacorb.util.Debug.output(4,"TAG_CODE_SETS found");			
+	    Debug.output(4,"TAG_CODE_SETS found");			
 
 	    // get server cs from IOR 
 	    CDRInputStream is =
-		new CDRInputStream(orb,pior.taggedComponents[i].component_data);
+		new CDRInputStream( orb, pior.taggedComponents[i].component_data);
+
 	    is.setLittleEndian(is.read_boolean());
 
-	    org.omg.CONV_FRAME.CodeSetComponentInfo inf;
-	    inf = org.omg.CONV_FRAME.CodeSetComponentInfoHelper.read(is);
+	    org.omg.CONV_FRAME.CodeSetComponentInfo inf = 
+		org.omg.CONV_FRAME.CodeSetComponentInfoHelper.read(is);
 		
 	    // char data, try UTF8 and ISO8859_1
 	    TCS = selectCodeSet(inf.ForCharData,CodeSet.UTF8);
-	    if(TCS == 0) 
+	    if( TCS == 0 ) 
 		TCS = CodeSet.ISO8859_1;
 			
 	    // wchar data, UTF8 or UTF16 can be used
 	    TCSW = selectCodeSet(inf.ForWcharData,CodeSet.UTF8);
-	    if(TCSW == 0) 
-		TCSW = selectCodeSet(inf.ForWcharData,CodeSet.UTF16);
+	    if( TCSW == 0) 
+		TCSW = selectCodeSet(inf.ForWcharData, CodeSet.UTF16);
 
-	    jacorb.util.Debug.output(4,"TCS selected: "+CodeSet.csName(TCS)+","+CodeSet.csName(TCSW));
+	    Debug.output(4,"TCS selected: "+CodeSet.csName(TCS)+","+
+                                     CodeSet.csName(TCSW));
 	    return true;
 	}
 
 	// if we are here then it means that no tagged component was in IOR, 
 	// probably some old server so set default parameters
+
 	// TODO(devik): we ends up here also for MICO server which seems to have some
-	//  strange IOR - have to investigate
+	// strange IOR - have to investigate
 
 	TCS = CodeSet.ISO8859_1;
 	TCSW = CodeSet.UTF16; // not as in OMG spec!
@@ -203,7 +285,8 @@ public final class Connection
 	// TODO(devik): Or should we ???
 	markTcsNegotiated();
 		
-	jacorb.util.Debug.output(4,"default TCS selected: "+CodeSet.csName(TCS)+","+CodeSet.csName(TCSW));
+	Debug.output(4,"default TCS selected: "+CodeSet.csName(TCS)+"," +
+                                 CodeSet.csName(TCSW));
 	return false;
     }
 	
@@ -212,23 +295,28 @@ public final class Connection
      * set TCS if one found. Returns true if TCS was correctly set.
      * Currently it's called from BasicAdapter.
      */
+
     public boolean setServerCodeSet(org.omg.IOP.ServiceContext [] ctx)
     {
-        if(!Environment.charsetScanCtx()) return false;
+        if( !Environment.charsetScanCtx() ) 
+	    return false;
+
 	// search all contexts until TAG_CODE_SETS found
-	for(int i=0;i<ctx.length;i++)
+	for( int i = 0; i < ctx.length; i++ )
 	{
-	    if(ctx[i].context_id != org.omg.IOP.TAG_CODE_SETS.value) continue;
+	    if( ctx[i].context_id != org.omg.IOP.TAG_CODE_SETS.value ) 
+		continue;
 			
 	    // TAG_CODE_SETS found, demarshall
-	    CDRInputStream is=new CDRInputStream(orb,ctx[i].context_data);
+	    CDRInputStream is = new CDRInputStream( orb, ctx[i].context_data);
 	    is.setLittleEndian(is.read_boolean());
-	    org.omg.CONV_FRAME.CodeSetContext csx;
-	    csx=org.omg.CONV_FRAME.CodeSetContextHelper.read(is);
-	    TCSW=csx.wchar_data;
-	    TCS=csx.char_data;
+	    org.omg.CONV_FRAME.CodeSetContext csx =
+		org.omg.CONV_FRAME.CodeSetContextHelper.read(is);
+	    TCSW = csx.wchar_data;
+	    TCS = csx.char_data;
 	    markTcsNegotiated();
-	    jacorb.util.Debug.output(4,"TCS set: "+CodeSet.csName(TCS)+","+CodeSet.csName(TCSW));
+	    Debug.output(4,"TCS set: "+CodeSet.csName(TCS)+","+
+                                     CodeSet.csName(TCSW));
 	    return true;
 	}
 	return false; // no TAG_CODE_SETS here
@@ -237,6 +325,7 @@ public final class Connection
     /**
      * Adds code set service context to another contexts if needed.
      */
+
     public org.omg.IOP.ServiceContext [] addCodeSetContext(org.omg.IOP.ServiceContext [] ctx,
 							   ParsedIOR pior)
     {		
@@ -249,79 +338,47 @@ public final class Connection
 		
 	// not negotiated but also TCS is not selected, so select one
 	// if it can't be selected (ior doesn't contain codesets) don't change ctx
-	if(TCS==0) 
+	if( TCS==0 ) 
+	{
 	    if(!selectCodeSet(pior)) 
 		return ctx;
-		
+	}
+
 	// encapsulate context
 
-	CDROutputStream os = new CDROutputStream(orb);
+	CDROutputStream os = new CDROutputStream( orb );
 	os.write_boolean(false);
 	org.omg.CONV_FRAME.CodeSetContextHelper.write(os,
-				     new org.omg.CONV_FRAME.CodeSetContext(TCS,TCSW));
+						      new org.omg.CONV_FRAME.CodeSetContext(TCS,TCSW));
 		
-	org.omg.IOP.ServiceContext [] ncx = new org.omg.IOP.ServiceContext[ctx.length+1];
+	org.omg.IOP.ServiceContext [] ncx =
+            new org.omg.IOP.ServiceContext[ctx.length+1];
+
 	System.arraycopy(ctx,0,ncx,0,ctx.length);
-	ncx[ctx.length]=new org.omg.IOP.ServiceContext(org.omg.IOP.TAG_CODE_SETS.value,
-						       os.getBufferCopy());
-	// jacorb.util.Debug.output(4,"TCS ctx added: "+CodeSet.csName(TCS)+","+CodeSet.csName(TCSW));
+	ncx[ctx.length] = 
+            new org.omg.IOP.ServiceContext(org.omg.IOP.TAG_CODE_SETS.value,
+                                           os.getBufferCopy());
+
+	// Debug.output(4,"TCS ctx added: "+CodeSet.csName(TCS)+","+CodeSet.csName(TCSW));
 	return ncx;
     }
 	
-    /**
-     * class Constructor, establishes a connection over a given socket
-     * this constructor is called from orb.getConnection for clients and
-     * from BasicAdapter for servers
-     *
-     * @param <code>Socket s</code>
-     * @param <code>boolean _is_server</code> - if opened from the server side.<br>
-     * @except <code>java.io.IOException</code>
-     */
-
-    public Connection(jacorb.orb.ORB orb, java.net.Socket s, boolean _is_server )
-	throws IOException
-    {
-	this.orb = orb;
-	mysock = s;
-	is_server = _is_server;
-        is_ssl     = jacorb.security.ssl.SSLSetup.isSSL ( s ); // bnv
-	in_stream = 
-	    new BufferedInputStream(mysock.getInputStream());
-	out_stream = 
-	    new BufferedOutputStream(mysock.getOutputStream(), Environment.outBufSize() );
-
-	String ip = mysock.getInetAddress().getHostAddress();
-	if( ip.indexOf('/') > 0)
-	    ip = ip.substring( ip.indexOf('/') + 1 );
-	String host_and_port = ip + ":"+ mysock.getPort();
-	String ssl = is_ssl ? "SSL " : ""; //bnv
-	connection_info = host_and_port;
-	client_count = 1;
-
-	if( ! _is_server ) // for clients only:
-	{
-	    jacorb.util.Debug.output(1,"New " + ssl + "connection to " + host_and_port);
-	    rep = new ReplyReceptor( this );
-	}
-	else
-	    jacorb.util.Debug.output(1,"Accepted " + ssl + "connection from " + host_and_port);
-    }
-
 
     /**
      *	Close a connection. Should only be done directly if the connection
      *  is broken or idCount has reached zero. Otherwise, use releaseConnection.
      */
 
-    synchronized void closeConnection()
+    public synchronized void closeConnection()
     {
-	jacorb.util.Debug.output(1,"Closing connection to " + connection_info);
+	Debug.output(1,"Closing connection to " + connection_info);
 	try
 	{
-	    if( rep != null )
-		rep.stopReceptor();
+	    if( repReceptor != null )
+		repReceptor.stopReceptor();
 
-	    jacorb.util.Debug.output(4,"Closing connection to " + connection_info + " (reply receptor closed)");
+	    Debug.output(4,"Closing connection to " + 
+                                     connection_info + " (reply receptor closed)");
 	    try 
 	    {
 		if( mysock != null )
@@ -329,38 +386,41 @@ public final class Connection
 	    } 
 	    catch(IOException e) 
 	    {
-		jacorb.util.Debug.output(4, e);
+		Debug.output(4, e);
 	    }
 	    
 	    if( in_stream != null )
 		in_stream.close();
 
-	    jacorb.util.Debug.output(4,"Closing connection to " + connection_info + " (in streams closed)");
+	    Debug.output(4,"Closing connection to " + 
+                                     connection_info + " (in streams closed)");
 
 	    if( out_stream != null )
 		out_stream.close();
 
-	jacorb.util.Debug.output(4,"Closing connection to " + connection_info + " (out streams closed)");
+	    Debug.output(4,"Closing connection to " + 
+                                     connection_info + " (out streams closed)");
 
 	} 
 	catch ( IOException e)
 	{
-	    jacorb.util.Debug.output(4,e);
+	    Debug.output(4,e);
 	}
 
 	in_stream = null;
 	out_stream = null;
-	rep = null;
+	repReceptor = null;
 
 	mysock = null;
 
-	jacorb.util.Debug.output(3,"Closing connection to " + connection_info + " (sockets closed)");
+	Debug.output(3,"Closing connection to " + 
+                                 connection_info + " (sockets closed)");
 
-	orb.removeConnection( this );
+	manager.removeConnection( this );
 
 	// connection_info = null;
 
-	if( buffers.size() > 0 )
+	if( replies.size() > 0 )
 	{
 	    try 
 	    {
@@ -368,20 +428,21 @@ public final class Connection
 	    } 
 	    catch ( org.omg.CORBA.COMM_FAILURE exe )
 	    {
-		jacorb.util.Debug.output(1,"Could not reconnect to " + 
-				   connection_info + " (loss of outstanding replies)");
+		Debug.output(1,"Could not reconnect to " + 
+					 connection_info + 
+                                         " (loss of outstanding replies)");
 	    }
 	}
     }
 
     public BufferManager getBufferManager()
     {
-	return ((jacorb.orb.ORB)orb).getBufferManager();
+        return orb.getBufferManager();
     }
 
     public boolean connected()
     {
-	return mysock != null;
+        return mysock != null;
     }
 
     /** 
@@ -390,19 +451,19 @@ public final class Connection
 
     public synchronized int getId()
     {
-	return id_count++; /* */
+        return id_count++; /* */
     }
 
     protected void incUsers()
     {
-	client_count++;
+        client_count++;
     }
 
     /** called by delegate when the delegate is duplicated */
 
     public void duplicate()
     {
-	incUsers();
+        incUsers();
     }
 
     /** 
@@ -413,29 +474,30 @@ public final class Connection
     private void abort()
 	throws  EOFException
     {
-	jacorb.util.Debug.output(3,"Connection to " + connection_info + " aborting...");
+	Debug.output(3,"Connection to " + 
+                                 connection_info + " aborting...");
 	Enumeration keys = replies.keys();
 	int lost_replies = 0;
 
 	while(keys.hasMoreElements()) 
 	{
-	  Object key = keys.nextElement();
+	    Object key = keys.nextElement();
 
-	  ReplyInputStream client = (ReplyInputStream)replies.get(key);
-	  client.cancel();
-	  lost_replies++;
+	    ReplyInputStream client = (ReplyInputStream)replies.get(key);
+	    client.cancel();
+	    lost_replies++;
 
-	  CDROutputStream os = (CDROutputStream) buffers.get(key);
-	  if (os != null)
-	    os.release();
+	    //  	  CDROutputStream os = (CDROutputStream) buffers.get(key);
+	    //  	  if (os != null)
+	    //  	    os.release();
 	}
 
 	replies.clear();
-	buffers.clear();
+	//	buffers.clear();
 	objects.clear();
 
 	if( lost_replies > 0 )
-	    jacorb.util.Debug.output(2,"Lost " + lost_replies + " outstanding replies");
+	    Debug.output(2,"Lost " + lost_replies + " outstanding replies");
 	throw new EOFException();
     }
 
@@ -444,7 +506,7 @@ public final class Connection
      * for this connection
      */
 
-    public  byte[] readBuffer() 
+    public byte[] readBuffer() 
 	throws IOException 
     {
 	for( int i = 0; i < Messages.MSG_HEADER_SIZE; i++ )
@@ -460,20 +522,18 @@ public final class Connection
 		/* if a client-side time out has been set we have to
 		   abort. NOTE: outstanding replies are lost! */
 		
-		jacorb.util.Debug.output(3,"Connection timed out");
+		Debug.output(3,"Connection timed out");
 	    }
 	    catch( Exception e )
 	    {
-		jacorb.util.Debug.output(3,e);
+		Debug.output(3,e);
 	    }
 	    if( input < 0 )		
 	    {
 		abort();
 	    }
 	    header[i]=(byte)input;
-	}
-
-	
+	}       
 
 	/* check for SECIOP vs. GIOP headers */
 
@@ -526,7 +586,7 @@ public final class Connection
 		} 
 		catch( IOException ie )
 		{
-		    jacorb.util.Debug.output(4,ie);
+		    Debug.output(4,ie);
 		}
 
 		if( n<0 )
@@ -539,8 +599,9 @@ public final class Connection
 	}
 	else 
 	{
-	    jacorb.util.Debug.output( 0, "Unknown message header type detected by " + 
-				      Thread.currentThread().getName() + ", discarding ", header );
+	    Debug.output( 0, "Unknown message header type detected by " + 
+				      Thread.currentThread().getName() + 
+                                      ", discarding ", header );
 	    return readBuffer();
 	    //	    throw new IOException("Unknown message header type!");
 	}
@@ -551,33 +612,35 @@ public final class Connection
      * and return an appropriate CDRInputStream
      */
 
-    public void  receiveReply() 
+    public void receiveReply() 
 	throws IOException, CloseConnectionException
     {
-	if( !connected())
+	if( !connected() )
 	    throw new EOFException();
+
 	byte [] buf = readBuffer();
+	int giop_version = buf[5];
 	int msg_type = buf[7];
 
+	Debug.output( 4, "received reply, GIOP version 1." + giop_version );
+
 	switch ( msg_type )
 	{
 	case  org.omg.GIOP.MsgType_1_0._Reply: 
 	    {
-		jacorb.util.Debug.output( 8, "receiveReply", buf );
+		Debug.output( 8, "receiveReply", buf );
 		Integer key = new Integer(Messages.getRequestId( buf, msg_type ));
 		org.omg.CORBA.Object o = (org.omg.CORBA.Object)objects.remove( key );
 
 		/* retrieve the ReplyInputStream that is waiting for this reply */
-		CDROutputStream os = (CDROutputStream) buffers.remove( key );
-		if (os != null)
-		  os.release();
 
 		ReplyInputStream pending = (ReplyInputStream)replies.remove( key );
       
 		if( pending != null ) 
 		    pending.init( buf, o );
 		else 
-		    System.err.println("Serious Error! No pending request for reply " + key );
+		    System.err.println("Serious Error! No pending request for reply " + 
+                                       key );
 		break;
 	    }
 	case org.omg.GIOP.MsgType_1_0._LocateReply:
@@ -585,9 +648,6 @@ public final class Connection
 		Integer key = new Integer(Messages.getRequestId( buf, msg_type ));
 
 		/* retrieve the ReplyInputStream that is waiting for this reply */
-		CDROutputStream os = (CDROutputStream) buffers.remove( key );
-		if (os != null)
-		  os.release();
 
 		jacorb.orb.giop.LocateReplyInputStream pending = 
 		    (jacorb.orb.giop.LocateReplyInputStream)replies.remove( key );
@@ -597,60 +657,88 @@ public final class Connection
 		    pending.init( buf );
 		} 
 		else 
-		    System.err.println("Serious Error! No pending request for reply " + key );
+		    System.err.println("Serious Error! No pending request for reply " + 
+                                       key );
 		break;
 	    }
+	case org.omg.GIOP.MsgType_1_0._MessageError:
+            {
+		Debug.output(1,"Got <MessageError> from server, something went wrong!");
+            }
 	case org.omg.GIOP.MsgType_1_0._CloseConnection:
 	    {
-		jacorb.util.Debug.output(3,"got close connection message");
+		Debug.output(3,"got close connection message");
 		throw new CloseConnectionException("Received <CloseConnection> from Server");
 	    }
 	default:
 	    throw new IOException("Received Message of unrecognizable type "+ msg_type);
 	}
-   }
+    }
+
 
     /**
      *
      */
-
+    
     public synchronized void reconnect()
 	throws org.omg.CORBA.COMM_FAILURE
     {	
-	jacorb.util.Debug.output(1,"Trying to reconnect to " + connection_info);
+	Debug.output(1,"Trying to reconnect to " + connection_info);
 
 	int retries = Environment.noOfRetries();
+
+	String host = connection_info.substring(0,connection_info.indexOf(":"));
+	int port = new Integer( connection_info.substring( connection_info.indexOf(":")+1)).intValue();
+
 	while( retries > 0 ) 
 	{
 	    try 
 	    {
-		mysock = new java.net.Socket( connection_info.substring(0,connection_info.indexOf(":")), 
-					      new Integer(connection_info.substring(
-							     connection_info.indexOf(":")+1)).intValue());
+                //noffke: by now, the factory knows if to provide
+                //ssl or not
+                mysock = socket_factory.createSocket( host, port );
+
 		mysock.setTcpNoDelay(true);
-		in_stream = new BufferedInputStream(mysock.getInputStream());
-		out_stream = new BufferedOutputStream(mysock.getOutputStream(), Environment.outBufSize());
+
+		in_stream = 
+		    new BufferedInputStream(mysock.getInputStream());
+
+		out_stream = 
+		    new BufferedOutputStream(mysock.getOutputStream(), 
+					     Environment.outBufSize());
 		
 		String ip = mysock.getInetAddress().getHostAddress();
+
 		if( ip.indexOf('/') > 0)
 		    ip = ip.substring( ip.indexOf('/') + 1 );
+
 		String host_and_port = ip + ":"+ mysock.getPort();
-		orb.addConnection( this );
+		manager.addConnection( this );
 		connection_info = host_and_port;
-		jacorb.util.Debug.output(1,"Reconnected to " + host_and_port);
-		rep = new ReplyReceptor( this );
-		for( Enumeration e = buffers.elements(); e.hasMoreElements();)
+
+		Debug.output(1,"Reconnected to " + host_and_port);
+
+		repReceptor = new ReplyReceptor( this );
+
+		// notify waiting clients that they ahve to retry
+		for( Enumeration e = replies.elements(); e.hasMoreElements();)
 		{
-		  CDROutputStream os = (CDROutputStream) e.nextElement();
-		  os.write_to(out_stream);
-		  os.release();
+		    ((ReplyInputStream) e.nextElement()).retry();
 		}
-		buffers.clear();
+
+		//  		for( Enumeration e = buffers.elements(); e.hasMoreElements();)
+		//  		{
+		//  		  CDROutputStream os = (CDROutputStream) e.nextElement();
+		//  		  os.write_to(out_stream);
+		//  		  os.release();
+		//  		}
+		//  		buffers.clear();
 		return;
 	    } 
 	    catch ( IOException c ) 
 	    { 
-		jacorb.util.Debug.output(1,"Retrying to reconnect to " + connection_info);
+		Debug.output(1,"Retrying to reconnect to " + 
+                                         connection_info);
 		try 
 		{
 		    Thread.sleep( Environment.retryInterval() );
@@ -671,14 +759,16 @@ public final class Connection
 
     public synchronized void releaseConnection()
     {
-	client_count--;
-	if( client_count == 0 )
-	{
-	    closeConnection();
-	}
-	else
-	    jacorb.util.Debug.output(2,"Releasing one connection to " + 
-			       connection_info );		
+        client_count--;
+        if( client_count == 0 )
+        {
+            closeConnection();
+        }
+        else
+        {
+            Debug.output(2,"Releasing one connection to " + 
+                                     connection_info );		
+        }
     }
 
     /**
@@ -691,8 +781,9 @@ public final class Connection
     }
 
 
-
-    /** writes the arguments of method calls to the output stream */
+    /** 
+     * writes the arguments of method calls to the output stream 
+     */
     
     public org.omg.CORBA.portable.InputStream sendRequest( org.omg.CORBA.Object o,
 							   RequestOutputStream os ) 
@@ -701,7 +792,7 @@ public final class Connection
 
 	if( !connected() )
 	{
-	    jacorb.util.Debug.output(3, "not connected" );
+	    Debug.output(3, "not connected" );
 	    reconnect();
 	}
 
@@ -711,51 +802,39 @@ public final class Connection
 	    
 	    try
 	    {
-//  		byte buf [];
-//  		int size = -1;
-
-		if( Environment.clientInterceptMessages())
-		{
-		    jacorb.util.Debug.output(4, " passing buffer to interceptors " );
-		    os.setBuffer(orb.client_messageIntercept_pre( o, os.getBufferCopy()));
-		}
-//  		else
-//  		{
-//  		    buf = os.getInternalBuffer();
-//  		    size = os.size();
-//  		}
-
 		if( os.response_expected() )
 		{
 		    rep = new ReplyInputStream(this, os.requestId());
 		    Integer key = new Integer( os.requestId() );
-		    buffers.put( key, os );
+		    //		    buffers.put( key, os );
 		    replies.put( key, rep );
 		    objects.put( key, o );
 		}
 
 		if (jacorb.util.Environment.verbosityLevel() > 4)
-		  //This is a costly op, since it involves buffer copying!
-		  jacorb.util.Debug.output(5,"send request", os.getBufferCopy());
+		{
+		    //This is a costly op, since it involves buffer copying!
+		    Debug.output(5,"send request", os.getBufferCopy());
+		}
 
 		os.write_to(out_stream);
 		
 		/** release the stream so its buffer can go back into the pool */
-		if (! os.response_expected() )
-		  os.release();		
+		os.release();		
 
-		//jacorb.util.Debug.output(5,"sendreq",buf,size);
+		//Debug.output(5,"sendreq",buf,size);
 	    } 
 	    catch ( Exception e )
 	    {
-		jacorb.util.Debug.output(2,e);
+		Debug.output(2,e);
 		throw new org.omg.CORBA.COMM_FAILURE();
 	    }		
 	    return rep;
 	}
     }
 
-    public synchronized LocateReplyInputStream sendLocateRequest( LocateRequestOutputStream os ) 
+    public synchronized LocateReplyInputStream sendLocateRequest( 
+                                                 LocateRequestOutputStream os ) 
 	throws org.omg.CORBA.COMM_FAILURE
     {
 	if( !connected() )
@@ -770,14 +849,15 @@ public final class Connection
 	    {
 		rep = new LocateReplyInputStream(this, os.requestId());
 		Integer key = new Integer( os.requestId() );
-		buffers.put( key, os );
+		//		buffers.put( key, os );
 		replies.put( key, rep );
 		
 		os.write_to(out_stream);
+		os.release();
 	    } 
 	    catch ( Exception e )
 	    {
-		jacorb.util.Debug.output(2,e);
+		Debug.output(2,e);
 		throw new org.omg.CORBA.COMM_FAILURE();
 	    }		
 	    return rep;
@@ -793,16 +873,16 @@ public final class Connection
 	throws org.omg.CORBA.COMM_FAILURE 
     {
 
-	    try
-	    {
-		out_stream.write( Messages.closeConnectionMessage());
-		out_stream.flush();
-		closeConnection();
-	    } 
-	    catch ( Exception e )
-	    {
-		throw new org.omg.CORBA.COMM_FAILURE();
-	    }
+	try
+	{
+	    out_stream.write( Messages.closeConnectionMessage());
+	    out_stream.flush();
+	    closeConnection();
+	} 
+	catch ( Exception e )
+	{
+	    throw new org.omg.CORBA.COMM_FAILURE();
+	}
 
     }
 
@@ -823,7 +903,7 @@ public final class Connection
 	    } 
 	    catch ( Exception e )
 	    {
-		jacorb.util.Debug.output(2,e);
+		Debug.output(2,e);
 		throw new org.omg.CORBA.COMM_FAILURE();
 	    }
 	}
@@ -841,21 +921,18 @@ public final class Connection
 	{
 	    try
 	    {	     
-		if( Environment.serverInterceptMessages())
+		if ( jacorb.util.Environment.verbosityLevel() > 4)
 		{
-		    byte[] buf = orb.server_messageIntercept_post(os.getBufferCopy());
-		    out_stream.write( buf, 0, buf.length);
-		    out_stream.flush();
+		    //This is a costly op, since it involves buffer copying!
+		    Debug.output(5,"send reply", os.getBufferCopy());
 		}
-		else
-		    // jacorb.util.Debug.output(3,"bypassing any message-level interceptors");
-		    os.write_to(out_stream);
-		
+
+		os.write_to(out_stream);		
 		os.release();
 	    } 
 	    catch ( Exception e )
 	    {
-		jacorb.util.Debug.output(2,e);
+		Debug.output(2,e);
 		throw new org.omg.CORBA.COMM_FAILURE();
 	    }
 	}
@@ -867,6 +944,8 @@ public final class Connection
     {
 	mysock.setSoTimeout(timeout);
     }
+
+
    
     /**
      * utility method, doesn't actually belong here
@@ -874,7 +953,7 @@ public final class Connection
     public static void dumpBA(byte bs[])
     {
 	int len = bs.length;
-	for (int i=0; i<len; i++)
+	for (int i = 0; i < len; i++)
 	{
 	    if (0 == i%20)
 	    {
@@ -903,19 +982,22 @@ public final class Connection
 	java.lang.System.out.print(c1+(c2+" ")+c3+"  ");
     }
 
-    public BufferedOutputStream get_out_stream(){
+    public BufferedOutputStream get_out_stream()
+    {
 	return out_stream;
     }
 
-    public Hashtable get_buffers(){
-	return buffers;
-    }
-	
-    public Hashtable get_replies(){
+    //      public Hashtable get_buffers(){
+    //  	return buffers;
+    //      }
+
+    public Hashtable get_replies()
+    {
 	return replies;
     }
 
-    public Hashtable get_objects(){
+    public Hashtable get_objects()
+    {
 	return objects;
     }
 
@@ -925,6 +1007,9 @@ public final class Connection
 	get_out_stream().write(buffer,0, len);
 	get_out_stream().flush();
     }
+
+
+
 }
 
 
